package vocleditor;

import org.eclipse.ui.plugin.*;
import org.osgi.framework.BundleContext;

import VOCLModel.*;

import java.util.*;

import vocleditor.VOCLEditorConstants;

/**
 * The main VOCLEditor plugin class to be used in the desktop.
 */
public class VOCLEditorPlugin extends AbstractUIPlugin {
	//The shared instance.
	private static VOCLEditorPlugin plugin;
	//Resource bundle.
	private ResourceBundle resourceBundle;
	
	/**
	 * The ID-String for the plugin
	 */
	public static final String ID = "VOCLEditor";
	
	/**
	 * The constructor.
	 */
	public VOCLEditorPlugin() {
		super();
		plugin = this;
	}

	/**
	 * This method is called upon plug-in activation
	 * @throws Exception
	 * @see AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
	}

	/**
	 * This method is called when the plug-in is stopped
	 * @throws Exception
	 * @see AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
		plugin = null;
		resourceBundle = null;
	}

	/**
	 * Returns the shared instance.
	 * @return the static instance of the plugin
	 */
	public static VOCLEditorPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 * @param key The key to search for
	 * @return the String with name 'key' or 'key' if not found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle =VOCLEditorPlugin.getDefault().getResourceBundle();
		try {
			return (bundle != null) ? bundle.getString(key) : key;
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle.
	 * @return null as it is not needed
	 */
	public ResourceBundle getResourceBundle() {
		return null;
	}
	
	/**
	 * Global method for output of debug-messages
	 * Will output the message if {@link VOCLEditorConstants#debugMode}
	 * is true
	 * @param x The message to ouput
	 */
	public static void printDebug(String x) {
		if(VOCLEditorConstants.debugMode)
			System.out.println("DEBUG: " + x);
	}
	
	/** 
	 * convenience method to find the Constraint a ModelElement belongs to
	 * (saves some links in the VOCLModel)
	 * @param melm a VOCLModel.ModelElement
	 * @return the (containing) Constraint or null
	 */
	public static Constraint getConstraint( ModelElement melm )
	{
		printDebug( "getConstraint: melm = " + (melm != null ? melm.getClass().getName() : "<null>!!!") );

		if ( melm instanceof Method )
			return getConstraint( ((Method) melm).getClassifierRole() );

		else if ( melm instanceof ClassifierRole )
			return getConstraint( ((ClassifierRole) melm).getNavExp() );

		else if ( melm instanceof VOCLExpression ) {
			VOCLExpression expr = (VOCLExpression) melm;
			if ( expr.getConstraint() == null )
				return getConstraint( (VOCLExpression) expr.eContainer() );
			return expr.getConstraint();
		}

		return null;
	}
	
	
	/**
	 * Returns all types in the constraint.
	 * @param constraint the constraint
	 * @return the list of types
	 */
	public static List getAllTypes(Constraint constraint){
		List types = new Vector();
		types.addAll(VOCLEditorConstants.getSimleTypes());
		List constraintTypes = getAllTypes((NavExp) constraint.getExpression());
		for(int i=0; i<constraintTypes.size(); i++){
			if(!types.contains(constraintTypes.get(i)))
				types.add(constraintTypes.get(i));
		}
		return types;
	}
	
	/**
	 * Returns all types in the navigation expression.
	 * @param navExp the navigation expression
	 * @return the list of types
	 */
	public static List getAllTypes(NavExp navExp){
		List types = new Vector();
		List classifierRoles = navExp.getClassifierRoles();
		for(int i=0; i<classifierRoles.size(); i++){
			ClassifierRole classifierRole = (ClassifierRole) classifierRoles.get(i);
			List classifierRoleTypes = getAllTypes(classifierRole);
			for(int j=0; j<classifierRoleTypes.size(); j++){
				if(!types.contains(classifierRoleTypes.get(j)))
					types.add(classifierRoleTypes.get(j));
			}
		}
		List expressions = navExp.getExpressions();
		for(int i=0; i<expressions.size(); i++){
			VOCLExpression expression = (VOCLExpression) expressions.get(i);
			List expressionTypes = getAllTypes(expression);
			for(int j=0; j<expressionTypes.size(); j++){
				if(!types.contains(expressionTypes.get(j)))
					types.add(expressionTypes.get(j));
			}
		}
		return types;
	}
	/**
	 * Returns all types in the navigation expression.
	 * @param expression the expression
	 * @return the list of types
	 */
	
	private static List getAllTypes(VOCLExpression expression){
		List types = new Vector();
		if(expression instanceof BoolExp){
			BoolExp boolExp = (BoolExp) expression;
			types.addAll(getAllTypes(boolExp.getOp1()));
			types.addAll(getAllTypes(boolExp.getOp2()));
		}else if(expression instanceof IfExp){
			IfExp ifExp = (IfExp) expression;
			types.addAll(getAllTypes(ifExp.getIfExp()));
			types.addAll(getAllTypes(ifExp.getThenExp()));
			types.addAll(getAllTypes(ifExp.getElseExp()));
		}else if(expression instanceof LetExp){
			LetExp letExp = (LetExp) expression;
			types.addAll(getAllTypes(letExp.getLet()));
			types.addAll(getAllTypes(letExp.getIn()));
		}else
			types.addAll(getAllTypes((NavExp) expression));
		return types;
	}
	
	/**
	 * Returns all types in the classifier role.
	 * @param classifierRole the classifier role
	 * @return the list of types
	 */
	public static List getAllTypes(ClassifierRole classifierRole){
		List types = new Vector();
		if(classifierRole.getType() != null && !types.contains(classifierRole.getType()))
			types.add(classifierRole.getType());
		List attributes = classifierRole.getAttributes();
		for(int j=0; j<attributes.size(); j++){
			Attribute attribute = (Attribute) attributes.get(j);
			if(!types.contains(attribute.getType()))
				types.add(attribute.getType());
		}
		if(classifierRole.getMethod() != null){
			Method method = classifierRole.getMethod();
			if(method.getReturnVariable() != null && method.getReturnVariable().getType() != null && !types.contains(method.getReturnVariable().getType()))
				types.add(method.getReturnVariable().getType());
			List parameters = method.getArguments();
			for(int j=0; j<parameters.size(); j++){
				VariableDeclaration varDecl = (VariableDeclaration) parameters.get(j);
				if(!types.contains(varDecl.getType()))
					types.add(varDecl.getType());
			}
		}
		if(classifierRole instanceof SetClassifierRole){
			SetClassifierRole setClassifierRole = (SetClassifierRole) classifierRole;
			if(setClassifierRole.getSize() != null && !types.contains(setClassifierRole.getSize().getType()))
				types.add(setClassifierRole.getSize().getType());
			if(setClassifierRole.getSum() != null && !types.contains(setClassifierRole.getSum().getType()))
				types.add(setClassifierRole.getSum().getType());
		}
		return types;
	}
	
	/**
	 * Returns the list of classifier role types in the constraint.
	 * @param constraint the constraint
	 * @return the list of classifier role types
	 */
	public static List getAllClassifierRoleTypes(Constraint constraint){
		return getAllClassifierRoleTypes((NavExp) constraint.getExpression());
	}
	
	/**
	 * Returns the list of classifer role types in the navigation expression.
	 * @param navExp the navigation expression
	 * @return the list of classifer role types
	 */
	public static List getAllClassifierRoleTypes(NavExp navExp){
		List types = new Vector();
		List classifierRoles = navExp.getClassifierRoles();
		for(int i=0; i<classifierRoles.size(); i++){
			ClassifierRole classifierRole = (ClassifierRole) classifierRoles.get(i);
			if(!types.contains(classifierRole.getType()))
				types.add(classifierRole.getType());
		}
		List expressions = navExp.getExpressions();
		for(int i=0; i<expressions.size(); i++){
			VOCLExpression expression = (VOCLExpression) expressions.get(i);
			List expressionTypes = getAllClassifierRoleTypes(expression);
			for(int j=0; j<expressionTypes.size(); j++){
				if(!types.contains(expressionTypes.get(j)))
					types.add(expressionTypes.get(j));
			}
		}
		return types;
	}
	/**
	 * Returns the list of classifer role types in the VOCL expression.
	 * @param expression the expression
	 * @return the list of classifer role types
	 */
	private static List getAllClassifierRoleTypes(VOCLExpression expression){
		List types = new Vector();
		if(expression instanceof BoolExp){
			BoolExp boolExp = (BoolExp) expression;
			types.addAll(getAllClassifierRoleTypes(boolExp.getOp1()));
			types.addAll(getAllClassifierRoleTypes(boolExp.getOp2()));
		}else if(expression instanceof IfExp){
			IfExp ifExp = (IfExp) expression;
			types.addAll(getAllClassifierRoleTypes(ifExp.getIfExp()));
			types.addAll(getAllClassifierRoleTypes(ifExp.getThenExp()));
			types.addAll(getAllClassifierRoleTypes(ifExp.getElseExp()));
		}else if(expression instanceof LetExp){
			LetExp letExp = (LetExp) expression;
			types.addAll(getAllClassifierRoleTypes(letExp.getLet()));
			types.addAll(getAllClassifierRoleTypes(letExp.getIn()));
		}else
			types.addAll(getAllClassifierRoleTypes((NavExp) expression));
		return types;
	}
   
	/**
	 * Returns all condition in the constraint.
	 * @param constraint the constraint
	 * @return the list of all condition
	 */
	public static List getAllConditions(Constraint constraint){
		return getAllConditions((NavExp) constraint.getExpression());
	}
	
	/**
	 * Returns all condition in the navigation expression.
	 * @param navExp the navigation expression
	 * @return the list all condition
	 */
	public static List getAllConditions(NavExp navExp){
		List conds = new Vector();
		if(navExp.getCondition() != null)
			conds.add(navExp.getCondition());
		
		List expressions = navExp.getExpressions();
		for(int i=0; i<expressions.size(); i++){
			VOCLExpression expression = (VOCLExpression) expressions.get(i);
			conds.addAll(getAllConditions(expression));
		}
		return conds;
	}
	/**
	 * Returns all condition in the navigation expression.
	 * @param expression the  VOCLexpression
	 * @return the list with all conditions
	 */
	private static List getAllConditions(VOCLExpression expression){
		List conds = new Vector();
		if(expression instanceof BoolExp){
			BoolExp boolExp = (BoolExp) expression;
			conds.addAll(getAllConditions(boolExp.getOp1()));
			conds.addAll(getAllConditions(boolExp.getOp2()));
		}else if(expression instanceof IfExp){
			IfExp ifExp = (IfExp) expression;
			conds.addAll(getAllConditions(ifExp.getIfExp()));
			conds.addAll(getAllConditions(ifExp.getThenExp()));
			conds.addAll(getAllConditions(ifExp.getElseExp()));
		}else if(expression instanceof LetExp){
			LetExp letExp = (LetExp) expression;
			conds.addAll(getAllConditions(letExp.getLet()));
			conds.addAll(getAllConditions(letExp.getIn()));
		}else
			conds.addAll(getAllConditions((NavExp) expression));
		return conds;
	}	
	
}
