/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.edit;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartListener;

import VOCLModel.ClassifierRole;
import VOCLModel.Context;
import VOCLModel.Method;
import VOCLModel.VOCLModelPackage;
import VOCLModel.VariableDeclaration;
import vocleditor.commands.EditContextCommand;
import vocleditor.dialog.ContextPropertiesDialog;
import vocleditor.figures.ContextFigure;

import vocleditor.VOCLEditorPlugin;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.widgets.Shell;

/**
 * The context edit part.
 * @author manaljav, bjoern
 */
public class ContextEditPart extends AbstractVOCLEditPart implements EditPartListener 
{	
	/**
	 * The constructor.
	 * @param context The Contex model element this edit part corresponds to
	 */
	public ContextEditPart( Context context )	{
		setModel( context );
		context.getConstraint().eAdapters().add( this );
		context.getConstraint().getExpression().eAdapters().add( this );
		addEditPartListener( this );
		notifyChanged( null );
	}
	
	/**
	 * Creates the figure
	 */
	protected IFigure createFigure() {	
		ContextFigure figure = new ContextFigure();
		figure.setText( getContextString());
		return figure;
	}

	/**
	 * Returns a string describing the context
	 */
	private String getContextString() {
		Context context = (Context) getModel();
		String text = "context ";
		
		if ( context.getType().equals( "undefined" ) )
			return text + " (yet undefined)";
		
		if( context.isIsClassifier() ) {
			if ( context.getContextClassifier() != null ) {
				String name = context.getContextClassifier().getName();
				if (!(name.equals("self")||name.equals("")))
				text += context.getContextClassifier().getName() + ": ";
				text += context.getContextClassifier().getType() + " ";
			}
		} else {
			if ( context.getContextMethod() != null ) {
				Method method = context.getContextMethod();
				if ( method.getClassifierRole() != null ) // only null during deletion 
					text += method.getClassifierRole().getType()  + "::";
				text += method.getName() + "(";
				for( int i = 0; i < method.getArguments().size(); i++ ) {
					text += ((VariableDeclaration) method.getArguments().get(i)).getName();
					if( i != method.getArguments().size() - 1 )
						text += ", ";
				}
				text += ") ";
			}
		}
		text += context.getType();
		if ( context.getConstraint().getName() != null )
			text += " " + context.getConstraint().getName();
		text += ":";
		
		return text;
	}

	/**
	 * Creates the name string of the context model element
	 * @return the context name (e.g. "peter : Person" or "Person::getAge()")
	 */
	protected String getContextName()
	{
		Context context = (Context) getModel();
		String contextName = "";
		if ( context.isIsClassifier() ) {
			ClassifierRole cr = context.getContextClassifier();
			if ( cr == null ) return "";  // happens while setting contextClassifier and contextMethod
			if (cr.getName() != null && !(cr.getName().equals("self")||cr.getName().equals("")) )
			contextName +=  cr.getName() + " : ";
			contextName += cr.getType() != null ? cr.getType() : "";
		} else {
			Method meth = context.getContextMethod();
			if ( meth == null ) return "";  // happens while setting contextClassifier and contextMethod
			ClassifierRole cr = meth.getClassifierRole();
			if ( cr == null ) return "";  // happens deleting contextMethod's classifier
			contextName += cr.getType() != null ? (cr.getType() + "::") : "";
			contextName += meth.getName() != null ? ( meth.getName() + "( ") : "";
			for ( int i = 0; i < meth.getArguments().size(); i++){
				contextName += ((VariableDeclaration) meth.getArguments().get(i)).getName();
				if( i != meth.getArguments().size() - 1)
					contextName += ", ";
			}
			contextName += ")";
	    }
		return contextName;
	}

	/**
	 * Does nothing, because we don't need editpolicies here.
	 */
	protected void createEditPolicies() {}
	
	/**
	 * Refreshes the figure after a model change.
	 * Model changes traced: attribute change of context Classifier or Method, removing them from their parents
	 */
	public void notifyChanged( Notification note ) 
	{
		Context context = (Context) getModel();
		
		if ( note != null && note.getNotifier() == context && 
			note.getFeatureID( VOCLModelPackage.class ) == VOCLModelPackage.MODEL_ELEMENT__NAME )
			return; // dont reach on context name changes

		VOCLEditorPlugin.printDebug( "Context:notifyChanged()" );
		// if not yet done: add this to the change listeners of the context thing (classifier or method)
		// to react on name changes and deletion
		ClassifierRole classifier = context.getContextClassifier();
		if ( classifier != null && ! classifier.eAdapters().contains( this ) )
			classifier.eAdapters().add( this );
		
		Method method = context.getContextMethod();
		if ( method != null ) {
			if ( ! method.eAdapters().contains( this ) )
				method.eAdapters().add( this );
			if ( ! method.getClassifierRole().eAdapters().contains( this ) )
				method.getClassifierRole().eAdapters().add( this );
		}

		ContextFigure figure = (ContextFigure) getFigure();
		figure.setText(getContextString());
		context.setName( getContextName() );
		refreshVisuals();
	}
	
	/**
	 * only for inheritance
	 */
	public void childAdded(EditPart child, int index) {}
	/**
	 * only for inheritance
	 */
	public void partActivated(EditPart editpart) {}
	/**
	 * only for inheritance
	 */
	public void partDeactivated(EditPart editpart) {}
	/**
	 * only for inheritance
	 */
	public void removingChild(EditPart child, int index) {}	

	/**
	 * (Un-)Highlights the AttributeFigure if the parts selection state changes.
	 */
	public void selectedStateChanged(EditPart editpart) {
		((ContextFigure) getFigure()).setSelected( getSelected() );
		refreshVisuals();
	}  

	/**
	 * Opens the properties dialog and creates and executes an
	 * {@link EditContextCommand} if necessary
	 */
	public void performRequest(Request request) {
		if(request.getType() == RequestConstants.REQ_OPEN) {
			Command command = null;
			AbstractVOCLEditPart editpart = (ContextEditPart) this;
			Context context = (Context)editpart.getModel();
			command = new EditContextCommand( context );
			ContextPropertiesDialog cpd = new ContextPropertiesDialog( new Shell(), context );
			cpd.open();
			if( cpd.getReturnCode() == Dialog.CANCEL)
				return;
			((EditContextCommand)command).setCommandAttributes(cpd.getContextName(), cpd.getContextType(),cpd.getIsClassifier(),cpd.getContextClassifier(), cpd.getContextMethod() );

			getRoot().getViewer().getEditDomain().getCommandStack().execute(command);
		}
	}
}
