/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.edit;



import java.util.List;
import java.util.Vector;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.requests.ReconnectRequest;
import org.eclipse.gef.requests.CreateConnectionRequest;

import VOCLModel.SetClassifierRole;
import VOCLModel.VOCLModelPackage;
import VOCLModel.ClassifierRole;
import VOCLModel.AssociationRole;
import VOCLModel.VariableDeclaration;
import vocleditor.figures.ClassifierFigure;
import vocleditor.figures.SetClassifierRoleFigure;
import vocleditor.model.ModelCreationFactory;
import vocleditor.commands.EditClassifierRoleCommand;
import vocleditor.commands.EditSetClassifierRoleCommand;
import vocleditor.dialog.ClassifierPropertiesDialog;
import vocleditor.dialog.SetClassifierRolePropertiesDialog;
import vocleditor.edit.VOCLAssociationRoleEditPart;

import vocleditor.VOCLEditorPlugin;

import org.eclipse.gef.RequestConstants;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.gef.commands.Command;

/**
 * EditPart for both ClassifierRoles and SetClassifierRoles. Only difference is in {@link #createFigure()}. 
 */
public class VOCLClassifierRoleEditPart extends AbstractVOCLEditPart implements NodeEditPart {
	
	/**
	 * Creates a new VOCLClassifierRoleEditPart instance.
	 * @param classifierRole the classifier role
	 */
	public VOCLClassifierRoleEditPart(ClassifierRole classifierRole){
		setModel(classifierRole);
	}

	/**
	 * Returns the classifier role.
	 * @return the classifier role
	 */
	public ClassifierRole getClassifierRole() {
		return (ClassifierRole)getModel();
	}
	
	/**
	 * Returns the list of source connections.
	 * @return the list of source connections
	 */
	protected List getModelSourceConnections() {
		return getClassifierRole().getSource();
	}
	
	/**
	 * Returns the list of target connections.
	 */
	protected List getModelTargetConnections() {
		return getClassifierRole().getTarget();
	}

	/**
	 * Returns the model children (Attributes) for this ClassifierRole.
	 * @return the model children (Attributes) for this ClassifierRole.
	 */
	protected List getModelChildren(){
 		return ((ClassifierRole) getModel()).getAttributes();
	}
	
	/** 
	 * Returns the classifier figure.
	 * @return the classifier figure
	 */
	protected ClassifierFigure getClassifierFigure(){
		if(getClassifierRole() instanceof SetClassifierRole){
			return ((SetClassifierRoleFigure) getFigure()).getClassifierFigure();
		}
		return (ClassifierFigure) getFigure();
	}

	/**
	 * Refreshs visuals, target-connections and so on if necessary.
	 */
	public void notifyChanged(Notification notification) {
		int featureId = notification.getFeatureID( VOCLModelPackage.class );
		VOCLEditorPlugin.printDebug( "CLASSROLEEDITPART:notify()");
		switch( featureId ) {
			case VOCLModelPackage.CLASSIFIER_ROLE__TARGET:
				refreshTargetConnections();
				break;		
			case VOCLModelPackage.CLASSIFIER_ROLE__SOURCE:
				refreshSourceConnections();
				break;
			case VOCLModelPackage.ATTRIBUTE:
			case VOCLModelPackage.CLASSIFIER_ROLE__NAME:
				updateVariable();
				refreshChildren();
				break;
			case 8:
				((NavExpEditPart) this.getParent()).refreshChildren();
				break;
		}
		if ( notification.getEventType() == Notification.ADD ) {
			refreshChildren();
		}
		if ( notification.getEventType() == Notification.REMOVE ) {
			refreshChildren();
		}
		refreshVisuals();
	}
	
	/**
	 * Creates, removes or updates the ClassiferRole's Variable
	 */
	protected void updateVariable()
	{
		ClassifierRole cr = (ClassifierRole) getModel();
		String name = cr.getName();
		if ( name == null || "".equals( name ) ) {
			if ( cr.getVariable() != null ) {
				VOCLEditorPlugin.getConstraint( cr ).getAllVariables().remove( cr.getVariable() );
				cr.setVariable( null );
			}
		} else {
			VariableDeclaration var = cr.getVariable();
			if ( var == null ) {
				var = (VariableDeclaration) (new ModelCreationFactory(VariableDeclaration.class)).getNewObject();
				cr.setVariable( var );
				VOCLEditorPlugin.getConstraint( cr ).getAllVariables().add( var );
			}
			cr.getVariable().setName( cr.getName() );
			cr.getVariable().setType( cr.getType() );
		}
	}
	
	/**
	 * Installs edit policies.
	 * @see VOCLClassifierRoleEditPolicy
	 * @see VOCLClassifierRoleComponentEditPolicy
	 */
	protected void createEditPolicies() {
		// install the edit policy to handle connection creation
		installEditPolicy( EditPolicy.GRAPHICAL_NODE_ROLE, new VOCLClassifierRoleEditPolicy() );
		// install the edit policy to handle deletion
		installEditPolicy( EditPolicy.COMPONENT_ROLE, new VOCLClassifierRoleComponentEditPolicy());
	}
	
	/**
	 * This function returns the ConnectionAnchor depending on {@link AssociationRole#getSourceAnchorAt()}.
	 * The anchors itsself are calculated in {@link ClassifierFigure#calculateAnchorPoints()}
	 */
	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) {
		return getClassifierFigure().getSourceConnectionAnchorAt((
				(AssociationRole)((VOCLAssociationRoleEditPart)connection).getModel()).getSourceAnchorAt()
			);
	}

	/**
	 * This one will determine the correct anchor number for the request's location
	 * via {@link ClassifierFigure#getSourceConnectionAnchorAt(Point)}
	 */
	public ConnectionAnchor getSourceConnectionAnchor(Request request) {
		if(request instanceof ReconnectRequest) {
			int anchorNo = getClassifierFigure().getSourceConnectionAnchorAt(((ReconnectRequest)request).getLocation());
			VOCLAssociationRoleEditPart edPart = (VOCLAssociationRoleEditPart)((ReconnectRequest)request).getConnectionEditPart();
			AssociationRole role = (AssociationRole)edPart.getModel();
			if(!edPart.isMoving) {
				edPart.lastAnchorAt = role.getSourceAnchorAt();
				edPart.isMoving = true;
			}
			role.setSourceAnchorAt(anchorNo);
			return getClassifierFigure().getSourceConnectionAnchorAt(anchorNo);
		}
		else if(request instanceof CreateConnectionRequest) {
			int anchorNo = getClassifierFigure().getSourceConnectionAnchorAt(((CreateConnectionRequest)request).getLocation());
			((AssociationRole)((CreateConnectionRequest)request).getNewObject()).setTargetAnchorAt(anchorNo);
			return getClassifierFigure().getSourceConnectionAnchorAt(anchorNo);
		} else
			return null;			
	}

	/**
	 * @see #getSourceConnectionAnchor(ConnectionEditPart)
	 */
	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {		
		return getClassifierFigure().getTargetConnectionAnchorAt((
				(AssociationRole)((VOCLAssociationRoleEditPart)connection).getModel()).getTargetAnchorAt()
			);
	}

	/**
	 * @see #getTargetConnectionAnchor(Request)
	 */
	public ConnectionAnchor getTargetConnectionAnchor(Request request) {
		if(request instanceof ReconnectRequest) {
			int anchorNo = getClassifierFigure().getTargetConnectionAnchorAt(((ReconnectRequest)request).getLocation());
			VOCLAssociationRoleEditPart edPart = (VOCLAssociationRoleEditPart)((ReconnectRequest)request).getConnectionEditPart();
			AssociationRole role = (AssociationRole)edPart.getModel();
			if(!edPart.isMoving) {
				edPart.lastAnchorAt = role.getTargetAnchorAt();
				edPart.isMoving = true;
			}
			role.setTargetAnchorAt(anchorNo);
			return getClassifierFigure().getTargetConnectionAnchorAt(anchorNo);
		}
		else if(request instanceof CreateConnectionRequest) {
			int anchorNo = getClassifierFigure().getTargetConnectionAnchorAt(((CreateConnectionRequest)request).getLocation());
			((AssociationRole)((CreateConnectionRequest)request).getNewObject()).setSourceAnchorAt(anchorNo);
			return getClassifierFigure().getTargetConnectionAnchorAt(anchorNo);
		} else
			return null;			
	}

	/**
	 * Refresh's visuals.
	 */
	public void refreshVisuals() {
		getClassifierFigure().setName( getClassifierRole().getName() + ": " + getClassifierRole().getType() );
			
		Point loc = new Point( getClassifierRole().getX(), getClassifierRole().getY() );
		Dimension size = new Dimension(-1,-1);
		Rectangle r = new Rectangle(loc ,size);
		((GraphicalEditPart) getParent()).setLayoutConstraint( this, getFigure(), r);
		
		if(getClassifierRole().getMethod() != null){
			List list = this.getParent().getChildren();
			for(int i=0; i<list.size(); i++){
				GraphicalEditPart editPart = (GraphicalEditPart) list.get(i);
				if(getClassifierRole().getMethod().equals(editPart.getModel()))
					((MethodEditPart) editPart).notifyChanged(null);
			}
		}
		
		if(getModel() instanceof SetClassifierRole){
			String sizeVariable = ((SetClassifierRole) getModel()).getSize() != null ? ((SetClassifierRole) getModel()).getSize().getName() : "";
			String iteratorVariable = ((SetClassifierRole) getModel()).getIterator() != null ? ((SetClassifierRole) getModel()).getIterator().getName() : "";
			String sumVariable = ((SetClassifierRole) getModel()).getSum() != null ? ((SetClassifierRole) getModel()).getSum().getName() : "";
			((SetClassifierRoleFigure) getFigure()).setOpKind(((SetClassifierRole) getModel()).getOpKind(), sizeVariable, iteratorVariable, sumVariable);
		}
	}
	
	/**
	 * Refresh source and target connections.
	 */
	public void refreshConnections() {
		ConnectionEditPart editPart;
		List connections = new Vector();
		connections.addAll(getTargetConnections());
		connections.addAll(getSourceConnections());
		for (int i = 0; i < connections.size(); i++) {
			editPart = (ConnectionEditPart)connections.get(i);
			editPart.refresh();
		}
	}

	/**
	 * Creates a {@link ClassifierFigure} or {@link SetClassifierRoleFigure}.
	 */
	protected IFigure createFigure() {
		if(getClassifierRole() instanceof SetClassifierRole)
			return new SetClassifierRoleFigure(this);
		return new ClassifierFigure(this);
	}
	
	/**
	 * Handles only double-clicks here and opens the corresponding properties-dialog.
	 */
	public void performRequest(Request request) {
		if(request.getType() == RequestConstants.REQ_OPEN) {
			Command command = null;
			if(getClassifierRole() instanceof SetClassifierRole) {
				command = new EditSetClassifierRoleCommand();
				((EditSetClassifierRoleCommand)command).setClassifierRole((SetClassifierRole)getClassifierRole());			
				SetClassifierRolePropertiesDialog cpd = new SetClassifierRolePropertiesDialog( new Shell(), VOCLEditorPlugin.getConstraint(getClassifierRole()), (SetClassifierRole)getClassifierRole());
				cpd.open();
				if( cpd.getReturnCode() == Dialog.CANCEL ) 
					return;
				((EditSetClassifierRoleCommand)command).setCommandAttributes(cpd.getClassifierName(), cpd.getClassifierType(), cpd.getOpKind(), cpd.getSizeVariable(), cpd.getIteratorVariable(), cpd.getSumVariable(), cpd.getChangedVariables(), cpd.getAttributes());
			} else {		
				command = new EditClassifierRoleCommand();
				((EditClassifierRoleCommand)command).setClassifierRole((ClassifierRole)getClassifierRole());			
				ClassifierPropertiesDialog cpd = new ClassifierPropertiesDialog( new Shell(), VOCLEditorPlugin.getConstraint(getClassifierRole()), (ClassifierRole)getClassifierRole());
				cpd.open();
				if( cpd.getReturnCode() == Dialog.CANCEL ) 
					return;
				((EditClassifierRoleCommand)command).setCommandAttributes(cpd.getClassifierName(),cpd.getClassifierType(),cpd.getChangedVariables(),cpd.getAttributes());
			}		
			
			getRoot().getViewer().getEditDomain().getCommandStack().execute(command);
		}
	}
}
