/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.figures;

import org.eclipse.draw2d.AbstractBorder;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Image;
import vocleditor.edit.VOCLConnectionAnchor;
import vocleditor.edit.VOCLClassifierRoleEditPart;

import vocleditor.VOCLEditorConstants;
import java.util.List;
import java.util.ArrayList;
import org.eclipse.draw2d.geometry.Point;

/**
 * Figure that represents a Classifier Role.
 * It has a label for the Classifier's name and type
 * and a compartment for the Classifier's attributes.
 */
public class ClassifierFigure
	extends RectangleFigure
{
	/**
	 * List of the incoming Anchorpoints
	 */
	protected List incomingConnectionAnchor;
	
	/**
	 * List of the outgoing Anchorpoints
	 */
	protected List outgoingConnectionAnchor;
	protected Label label;
	protected ToolbarLayout layout;
	private CompartmentFigure attributeCompartment = new CompartmentFigure();
	
	VOCLClassifierRoleEditPart edPart;
		
	/**
	 * Create a new ClassifierFigure.
	 * 
	 * @param theEdPart the editpart that creates this figure
	 */
	public ClassifierFigure(VOCLClassifierRoleEditPart theEdPart) {
		layout = new ToolbarLayout();
	    setLayoutManager(layout);
	    setBorder(new LineBorder(ColorConstants.black,1));
	    setBackgroundColor( VOCLEditorConstants.CLASSIFIER_COLOR );
	    setOpaque(true);
	    label = new Label(" ");
	    label.setIcon( new Image( null, ClassifierFigure.class.getResourceAsStream( "icons/classifier_role.gif" )));
	    add( label, 0 );
	    layout.setMinorAlignment(ToolbarLayout.ALIGN_TOPLEFT);
	    edPart = theEdPart;		
	 }
	
 
	/** 
	 * Adds AttributeFigures to the attribute compartment or behaves like its super method otherwise.
	 * @see org.eclipse.draw2d.IFigure#add(org.eclipse.draw2d.IFigure, java.lang.Object, int)
	 */
	public void add(IFigure figure, Object constraint, int index) 
	{	
		if ( figure instanceof AttributeFigure ) {
			if ( ! getChildren().contains( attributeCompartment ) )
				add(attributeCompartment);
			attributeCompartment.add(figure, constraint, index); }
		else
			super.add(figure, constraint, index);
	}
	
	/**
	 * Removes AttributeFigures from the attribute compartment or behaves like its super method otherwise.
	 * @see org.eclipse.draw2d.IFigure#remove(org.eclipse.draw2d.IFigure)
	 */
	public void remove(IFigure figure) {
		if ( figure instanceof AttributeFigure ) {
			attributeCompartment.remove(figure);
			if ( attributeCompartment.getChildren().isEmpty() )
				super.remove( attributeCompartment );
		} else
			super.remove(figure);
	}

	/**
	 * Redirects call to the attribute compartment for AttributeFigures or behaves like its super method otherwise.
	 * @see org.eclipse.draw2d.IFigure#setConstraint(org.eclipse.draw2d.IFigure, java.lang.Object)
	 */
	public void setConstraint(IFigure child, Object constraint) 
	{
		if ( child instanceof AttributeFigure )
			attributeCompartment.setConstraint( child, constraint );
		else
			super.setConstraint(child, constraint);
	}
	
	/**
	 * set the classifier name to be displayed.
	 * @param name the classifier name to be displayed 
	 */
	public void setName(String name){
		label.setText(name);
		validate();
	}
	
	/**
	 * Adjusts the size of this figure and refreshes anchor points.
	 * @see org.eclipse.draw2d.IFigure#validate()
	 */
	public void validate(){
		if ( isValid() ) return;
		layout.setConstraint(label, new Rectangle(5,5,-1,-1));
		calculateAnchorPoints();
		edPart.refreshConnections();
		super.validate();
	}
	
	/**
	 * returns a list of source connections
	 * @return a list of source connections
	 */
	public List getSourceConnectionAnchors(){
		return outgoingConnectionAnchor;
	}

	/**
	 * returns a list of target connections
	 * @return a list of target connections
	 */	
	public List getTargetConnectionAnchors(){
		return incomingConnectionAnchor;
	}

	/**
	 * returns the index of the source connection closest to a given point.
	 * 
	 * @param p a point
	 * @return the index of the source connection closest to p
	 */	
	public int getSourceConnectionAnchorAt(Point p) {
		return getClosestConnectionAnchorAt(p, getSourceConnectionAnchors());
	}

	/**
	 * returns the index of the target connection closest to a given point.
	 * 
	 * @param p a point
	 * @return the index of the target connection closest to p
	 */	
	public int getTargetConnectionAnchorAt(Point p) {
		return getClosestConnectionAnchorAt(p, getTargetConnectionAnchors());
	}

	/**
	 * returns the source VOCLConnectionAnchor at the given index.
	 * @param index index of the connection source anchor point
	 * @return the source VOCLConnectionAnchor at the given index.
	 */	
	public VOCLConnectionAnchor getSourceConnectionAnchorAt(int index) {
		return (VOCLConnectionAnchor)getSourceConnectionAnchors().get(index);
	}

	/**
	 * returns the target VOCLConnectionAnchor at the given index.
	 * @param index index of the connection target anchor point
	 * @return the target VOCLConnectionAnchor at the given index.
	 */	
	public VOCLConnectionAnchor getTargetConnectionAnchorAt(int index) {
		return (VOCLConnectionAnchor)getTargetConnectionAnchors().get(index);
	}

	/**
	 * returns the index of the anchor point closest to a given point.
	 * 
	 * @param p the point 
	 * @param connectionAnchors the connection anchors to choose one from
	 * @return the index of the anchor closest to p
	 */
	static public int getClosestConnectionAnchorAt( Point p, List connectionAnchors) {
		long min = Long.MAX_VALUE;
		int found = 0;
		
		for (int i = 0; i < 8; i++) {
			VOCLConnectionAnchor c = (VOCLConnectionAnchor) connectionAnchors.get(i);
			Point p2 = c.getLocation(null);
			long d = p.getDistance2(p2);
			if (d < min) {
				min = d;
				found = i;
			}
		}
		return found;
	}
	
	/**
	 * Says that this figure uses local coordinates
	 * @see org.eclipse.draw2d.Figure#useLocalCoordinates()
	 */
	protected boolean useLocalCoordinates() {
		return true;
	}

	
	/**
	 * returns the minimum size of this figure
	 * @see org.eclipse.draw2d.IFigure#getMinimumSize(int, int)
	 */
	public Dimension getMinimumSize(int wHint, int hHint) {
		
		return getPreferredSize();
	}

	/**
	 * Returns the preferred size of this figure.
	 * The preferred size is based on the size of the label
	 * and the attribute compartment.
	 * 
	 * @see org.eclipse.draw2d.IFigure#getPreferredSize(int, int)
	 * @return a Dimension object holding the preferred size
	 */
	public Dimension getPreferredSize(int wHint, int hHint) {
		int width, height;
		
		Dimension labelDim = label.getPreferredSize();
		Dimension afDim    = attributeCompartment.getPreferredSize();

		width  = Math.max( labelDim.width + 10, 50 );
		width  = Math.max( afDim.width + 10, width );
		height = Math.max( afDim.height + labelDim.height, 22 );
		
		return new Dimension( width, height );
	}

	/**
	 * Calculates possible Anchorpoints
	 *
	 */
	private void calculateAnchorPoints() {
		incomingConnectionAnchor = new ArrayList();
		outgoingConnectionAnchor = new ArrayList();
		
		VOCLConnectionAnchor tempA = new VOCLConnectionAnchor(this);
		tempA.setOffset(0, 0); 
		incomingConnectionAnchor.add(tempA);
		outgoingConnectionAnchor.add(tempA);  

//		 mitte oben
		tempA = new VOCLConnectionAnchor(this);
		tempA.setOffset(this.getSize().width/2, 0);
		incomingConnectionAnchor.add(tempA);
		outgoingConnectionAnchor.add(tempA);

//		 rechts oben 
		tempA = new VOCLConnectionAnchor(this);
		tempA.setOffset(this.getSize().width - 1, 0);
		incomingConnectionAnchor.add(tempA);
		outgoingConnectionAnchor.add(tempA);

//		 mitte links
		tempA = new VOCLConnectionAnchor(this);
		tempA.setOffset(0, this.getSize().height/2);
		incomingConnectionAnchor.add(tempA);
		outgoingConnectionAnchor.add(tempA);

//		 mitte rechts 
		tempA = new VOCLConnectionAnchor(this);
		tempA.setOffset(this.getSize().width -1, this.getSize().height/2);
		incomingConnectionAnchor.add(tempA);
		outgoingConnectionAnchor.add(tempA);

//		 unten links
		tempA = new VOCLConnectionAnchor(this);
		tempA.setOffset(0, this.getSize().height - 1);
		incomingConnectionAnchor.add(tempA);
		outgoingConnectionAnchor.add(tempA);

//		 unten mitte
		tempA = new VOCLConnectionAnchor(this);
		tempA.setOffset(this.getSize().width/2, this.getSize().height -1);
		incomingConnectionAnchor.add(tempA);
		outgoingConnectionAnchor.add(tempA);

//		 unten rechts
		tempA = new VOCLConnectionAnchor(this);
		tempA.setOffset(this.getSize().width -1, this.getSize().height -1);
		incomingConnectionAnchor.add(tempA);
		outgoingConnectionAnchor.add(tempA);
	}
	
	/**
	 * a figure used to hold a classifiers attribute figures
	 * @author bjoern
	 */
	public static class CompartmentFigure extends Figure 
	{
		/** 
		 * constructs a new CompartmentFigure
		 *
		 */
		public CompartmentFigure() 
		{
		    ToolbarLayout layout = new ToolbarLayout( false );
		    layout.setMinorAlignment(ToolbarLayout.ALIGN_TOPLEFT);
		    layout.setStretchMinorAxis(true);
		    layout.setSpacing(2);
		    setBorder(new CompartmentFigureBorder());
		    setLayoutManager(layout);
		}
		  
		/**
		 * a border for the CompartmentFigure
		 *
		 */
		public class CompartmentFigureBorder extends AbstractBorder 
		{
			/**
			 * returns the insets of this border
			 * @param figure the figure to calculate the insets for
			 * @return the insets of this border
			 */
		    public Insets getInsets(IFigure figure) {
		    	return new Insets(3,3,3,3);
		    }
		    
		    /**
		     * paints the border line
		     * 
		     * @param figure the figure to paint this border onto
		     * @param graphics a graphics object used to draw
		     * @param insets formerly calclated insets
		     */
		    public void paint(IFigure figure, Graphics graphics, Insets insets) {
		    	graphics.drawLine(getPaintRectangle(figure, insets).getTopLeft(),
		                        tempRect.getTopRight());
		    }
		}	  
	}

}