/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.figures;


import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.RectangleFigure;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.swt.graphics.Color;

/**
 * The graphical representation of an if expression.
 * It consist of an if part and a then-else part (which is a SplitFigure).
 * The figure can be split horizontally or vertically and the percentage of
 * space used by the if-part can be set.
 * 
 * @author bjoern
 */
public class IfExpFigure extends RectangleFigure 
{
	ToolbarLayout layout;
	int splitPercentage = 50;
	boolean horizontal = false;
	LabeledFigure ifFigure;
	SplitFigure thenElseFigure;
	
	/**
	 * Constructs a new IfExpFigure
	 * 
	 * @param fillColor the background color to be used within the figure
	 * @param outsideColor the color to be used for the appearing like outside the figure
	 */
	public IfExpFigure( Color fillColor, Color outsideColor ) 
	{
		layout = new ToolbarLayout();
	    setLayoutManager( layout );
	    setBackgroundColor( fillColor );
	    setOutline( false );
	    NegatableBorder border = new NegatableBorder();
	    border.setBackground( outsideColor );
		setBorder( border );	
		setOpaque( true );
		ifFigure = new LabeledFigure( "if", fillColor, outsideColor );
		thenElseFigure = new SplitFigure( true, fillColor, fillColor );
		thenElseFigure.setTopTitle( "then" );
		thenElseFigure.setMiddleTitle( "else" );
		add( ifFigure );
		add( thenElseFigure );
	    layout.setMinorAlignment(ToolbarLayout.ALIGN_TOPLEFT);
	}

	/**
	 * Sets whether this figure should be split horizontally or not.
	 * @param horizontal if true the if part is left of the then-else part, otherwise the if part is above the then-else part
	 */
	public void setHorizontal( boolean horizontal )
	{
		this.horizontal = horizontal;
		layout.setVertical( ! horizontal );
		thenElseFigure.setHorizontal( ! horizontal );
		if ( horizontal ) {
			ifFigure.labelBorder.setTextPos( PositionConstants.LEFT );
			ifFigure.labelBorder.setLines( true, true, true, false, false, false );
		}
		else {
			ifFigure.labelBorder.setTextPos( PositionConstants.TOP );
			ifFigure.labelBorder.setLines( true, false, true, true, false, false );
		}
		invalidateTree();
		revalidate();
	}

	/**
	 * Sets the percentage of space used by the if-part. 
	 * 
	 * @param split percentage of space used by the if-part
	 */
	public void setSplitPercentage1( int split )
	{
		splitPercentage = split;
		invalidateTree();
		revalidate();
	}

	/**
	 * Sets the percentage of space used by the then-part of the then-else part. 
	 * 
	 * @param split percentage of space used by the then-part
	 */
	public void setSplitPercentage2( int split )
	{
		thenElseFigure.setSplitPercentage( split );
	}

	
	/**
	 * Redefined to add NavExp figures to the if-part or to the then-else part of this figure.
	 * @see org.eclipse.draw2d.IFigure#add(org.eclipse.draw2d.IFigure, java.lang.Object, int)
	 */
	public void add(IFigure figure, Object constraint, int index) 
	{	
		if ( figure == ifFigure || figure == thenElseFigure )
			super.add( figure, constraint, index );
		else if ( ifFigure.getChildren().size() == 0 )
			ifFigure.add( figure, constraint, ifFigure.getChildren().size() );
		else 
			thenElseFigure.add( figure, constraint, thenElseFigure.getChildren().size() );		
	}
	
	/**
	 * redefined to set size if the parts based on whether this figure is split horizontally or not
	 * @see org.eclipse.draw2d.IFigure#validate()
	 */
	public void validate() 
	{
		if ( isValid() ) return;
		super.validate();

		int w = getBounds().width + 1;
		int h = getBounds().height + 1;

		if ( horizontal ) {
			ifFigure.setPreferredSize( new Dimension( (w * splitPercentage) / 100, h ) );
			thenElseFigure.setPreferredSize( new Dimension( (w * (100-splitPercentage)) / 100, h ) );
		} else {
			ifFigure.setPreferredSize( new Dimension( w, (h * splitPercentage) / 100 ) );
			thenElseFigure.setPreferredSize( new Dimension( w, (h * (100-splitPercentage)) / 100 ) );
		}
	}
	
	/**
	 * Says that this figure uses local coordinates.
	 * @see org.eclipse.draw2d.Figure#useLocalCoordinates()
	 */
	protected boolean useLocalCoordinates() {
		return true;
	}
	

	/**
	 * Returns the minimum size of this Figure
	 * based on the size of the if part and the then-else part.
	 * @see org.eclipse.draw2d.IFigure#getMinimumSize(int, int)
	 */
	public Dimension getMinimumSize(int wHint, int hHint) 
	{
		Dimension d;
		
		Dimension dim1 = ifFigure.getMinimumSize();
		Dimension dim2 = thenElseFigure.getMinimumSize();
		if ( horizontal )
			d = new Dimension( dim1.width + dim2.width, Math.max( dim1.height, dim2.height ) );
		else
			d = new Dimension( Math.max( dim1.width, dim2.width), dim1.height + dim2.height );

		return d;
	}

	/**
	 * returns the preferred size of this figure
	 * @see org.eclipse.draw2d.IFigure#getPreferredSize(int, int)
	 */
	public Dimension getPreferredSize(int wHint, int hHint) 
	{
		return getMinimumSize( wHint, hHint );
	}	
}