package vocleditor.figures;

import org.eclipse.draw2d.AbstractLabeledBorder;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.FigureUtilities;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Insets;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;

import vocleditor.VOCLEditorConstants;

/**
 * A labeled border intended to house a Figure with a group of children. 
 * The label should serve as a description of the group.
 */
public class LabelBorder
	extends AbstractLabeledBorder
{	
private Color fillColor;
private Color outsideColor;
private int textPos = PositionConstants.TOP;
private Dimension titleBounds = null;
private Dimension labelBounds = null;
private Dimension charSizes[] = null;
private boolean leftLine, rightLine, topLine, bottomLine;
private boolean leftComplete;
private boolean rightComplete;
private int textAlignment = PositionConstants.LEFT;
private Insets padding = new Insets( 0, 0, 0, 0);
private String title;

/**
 * Constructs a TitleBarBorder with its label set to the name of this class.
 * @param fill the {@link Color} to fill the border with
 * @param outside the {@link Color} of the outside border
 * @since 2.0
 */
public LabelBorder( Color fill, Color outside ) 
{
	this.fillColor = fill;
	this.outsideColor = outside;
	leftLine = rightLine = topLine = bottomLine = true;
	leftComplete = rightComplete = false;
}

/**
 * Sets whichs lines o this border should be drawn
 * @param top draw the top line 
 * @param bottom draw the bottom line
 * @param left draw the left line
 * @param right draw the right line
 * @param rightComplete draw the right (or top) line beyond the line at the middle of the title text
 * @param leftComplete draw the left (or bottom) line beyond the line at the middle of the title text
 */
public void setLines( boolean top, boolean bottom, boolean left, boolean right, boolean rightComplete, boolean leftComplete )
{
	this.leftComplete = leftComplete;
	this.rightComplete = rightComplete;
	this.topLine = top;
	this.bottomLine = bottom;
	this.leftLine = left;
	this.rightLine = right;
}

/**
 * Sets the position od
 * @param textPos
 */
public void setTextPos( int textPos )
{
	this.textPos    = textPos;
	this.titleBounds = null;
	this.charSizes  = null;
}

final private boolean hasLabel() 
{
	return getLabel() != null && ! "".equals( getLabel() );
}

final private boolean hasTitle() 
{
	return title != null && ! "".equals( title );
}

/**
 * Calculates and returns the Insets for this border. 
 * 
 * @param figure the figure on which Insets calculations are based
 * @return the calculated Insets
 * @since 2.0
 */
protected Insets calculateInsets(IFigure figure) 
{
	Insets ins = new Insets( 1, 1, 1, 1 );

	if ( hasLabel() )
		ins.top += 10;
	if ( hasTitle() && textPos == PositionConstants.LEFT )
		ins.left += 8;
	else if ( hasTitle() )
		ins.top += 10;
	return ins;
}

/**
 * Returns the background Color of this LabelBorder.
 * @return the background color
 * @since 2.0
 */
protected Color getBackgroundColor() {
	return fillColor;
}

/**
 * Says that this border does not use local coordinates.
 * @see AbstractLabeledBorder#isOpaque()
 */
public boolean isOpaque() {
	return false;
}

/**
 * Paints this border.
 * @see Border#paint(IFigure, Graphics, Insets)
 */
public void paint(IFigure figure, Graphics g, Insets insets) 
{
	if ( titleBounds == null ) 
		calcTextSizes( (Figure) figure );

	tempRect.setBounds(getPaintRectangle(figure, insets));
	Rectangle bounds = tempRect;
	g.translate( bounds.x, bounds.y );
	bounds.x = bounds.y = 0;
	g.clipRect( bounds );
	g.setFont(getFont(figure));

	if ( ! hasTitle() || titleBounds == null ) 
		paintNoText( g, bounds );
	else if ( textPos == PositionConstants.LEFT )
		paintLeft( g, bounds );
	else
		paintTop( g, bounds );
}

/**
 * paints the figure if title is set to be left
 * @param g graphics
 * @param bounds size/position of figure
 */
protected void paintLeft( Graphics g, Rectangle bounds )
{
	int rx = titleBounds.width / 2;
	int ty = (bounds.height - titleBounds.height) / 2;

	g.setBackgroundColor( outsideColor );
	g.fillRectangle( 0, 0, rx, bounds.height);

	g.setForegroundColor( ColorConstants.black );
	if ( topLine && rightComplete )
		g.drawLine( 0, 0, bounds.width, 0);
	if ( topLine && ! rightComplete )
		g.drawLine( rx, 0, bounds.width, 0);
	if ( bottomLine && leftComplete )
		g.drawLine( 0, bounds.height-1, bounds.width, bounds.height-1 );
	if ( bottomLine && ! leftComplete )
		g.drawLine( rx, bounds.height-1, bounds.width, bounds.height-1 );
	if ( rightLine )
		g.drawLine( bounds.width-1, 0, bounds.width-1, bounds.height );

	g.setForegroundColor( ColorConstants.black );
	g.drawLine( rx, 0, rx, ty - 5);
	for ( int i = 0; i < title.length(); i++ ) {
		g.setForegroundColor( ColorConstants.black );
		g.drawString( ""+title.charAt(i), (titleBounds.width - charSizes[i].width)/2 + 1, ty + 1 );
		g.setForegroundColor( VOCLEditorConstants.EXPRESSION_TEXT_COLOR );
		g.drawString( "" + title.charAt(i), (titleBounds.width - charSizes[i].width)/2, ty );
		ty += charSizes[i].height;
	}
	g.setForegroundColor( ColorConstants.black );
	g.drawLine( rx, ty+5, rx, bounds.height );
	if ( hasLabel() )
		g.drawString( getLabel(), 5 + titleBounds.width, 2 ); 
}

/**
 * paints the figure if title is set to be on top
 * @param g graphics
 * @param bounds size/position of figure
 */
protected void paintTop( Graphics g, Rectangle bounds )
{
	int ry = titleBounds.height / 2 - 3;
	int tx = (bounds.width - titleBounds.width) / 2;

	g.setBackgroundColor( outsideColor );
	g.fillRectangle( 0, 0, bounds.width + 2, ry );

	g.setForegroundColor( ColorConstants.black );
	if ( rightLine && rightComplete )
		g.drawLine( bounds.width-1, 0, bounds.width-1, bounds.height );
	if ( rightLine && ! rightComplete )
		g.drawLine( bounds.width-1, ry, bounds.width-1, bounds.height );
	if ( leftLine && leftComplete )
		g.drawLine( 0, 0, 0, bounds.height-1 );
	if ( leftLine && ! leftComplete )
		g.drawLine( 0, ry, 0, bounds.height-1 );
	if ( bottomLine )
		g.drawLine( 0, bounds.height-1, bounds.width, bounds.height-1 );
	
	
	g.setForegroundColor( ColorConstants.black );
	g.drawString( title, tx + 1, -1) ;
	g.setForegroundColor( VOCLEditorConstants.EXPRESSION_TEXT_COLOR );
	g.drawString( title, tx, -2 );
	g.setForegroundColor( ColorConstants.black );
	g.drawLine( 0, ry, tx - 5, ry );
	g.drawLine( tx + titleBounds.width + 10, ry, bounds.width, ry );
	if ( hasLabel() )
		g.drawString( getLabel(), 5, titleBounds.height - 10 ); 
}	

/**
 * paints the figure if no title is set
 * @param g graphics
 * @param bounds size/position of figure
 */
private int paintNoText( Graphics g, Rectangle bounds )
{
	g.setForegroundColor( ColorConstants.black );
	if ( topLine )
		g.drawLine( 0, 0, bounds.width, 0);
	if ( bottomLine )
		g.drawLine( 0, bounds.height-1, bounds.width, bounds.height-1 );
	if ( leftLine )
		g.drawLine( 0, 0, 0, bounds.height );
	if ( rightLine )
		g.drawLine( bounds.width-1, 0, bounds.width-1, bounds.height );
	if ( hasLabel() )
		g.drawString( getLabel(), 5, 2 ); 
	return 1;
}

/**
 * calcs label and title sizes based on font and text string
 * @param f a figure
 */
protected void calcTextSizes( Figure f )
{
	Font font = f.getFont();
	titleBounds = null;
	if ( title != null && font != null && ! "".equals( title )) {
		if ( textPos == PositionConstants.TOP )
			titleBounds = FigureUtilities.getTextExtents( title, font );
		else {
			Dimension d;
			titleBounds = new Dimension( 0, 0 );
			charSizes = new Dimension[ title.length() ];
			for ( int i = 0; i < title.length(); i++ ) {
				d = FigureUtilities.getTextExtents( ""+title.charAt(i), font );
				charSizes[i] = new Dimension( d.width, d.height );
				charSizes[i].height -= 2; // looks better here - but does it always work??
				if ( d.width > titleBounds.width ) titleBounds.width = d.width;
				titleBounds.height += charSizes[i].height;
			}
		}
	} 
	if ( hasLabel() )
		labelBounds = FigureUtilities.getTextExtents( getLabel(), font );
	invalidate();
}

/**
 * Sets the background color of the area within the boundaries of this border. This is 
 * required as this border takes responsibility for filling up the region, as 
 * TitleBarBorders are always opaque.
 *
 * @param color the background color
 * @since 2.0
 */
public void setBackgroundColor(Color color) {
	fillColor = color;
}

/**
 * returns the title string
 * @return the title string
 */
public String getTitle() 
{
	return title;
}

/**
 * sets the title string
 * @param s the new title string
 */
public void setTitle( String s ) 
{
	if ( s == null || "".equals( s ) )
		title = null;
	else
		title = s;
}


}