/*
 * VOCLEditor PlugIn for Eclipse
 *
 * Gruppe 4 - Sommersemester 2005  
 */
package vocleditor.model;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.core.runtime.IPath;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import VOCLModel.VOCLDiagram;
import VOCLModel.VOCLModelFactory;
import VOCLModel.VOCLModelPackage;
import VOCLModel.impl.VOCLModelPackageImpl;

/**
 * Drives the model. Acts as the model entry point.
 */
public class VOCLModelManager {
	/**
	 * For the purpose of the simple editor, a file can only contain a VOCLModel.
	 * In EMF, a resource provides the way to have access to the model content.
	 */
	private Resource resource = null;

	/**
	 * Contains the factory associated with the model.
	 */
	private static VOCLModelFactory voclmodelFactory = null;

	/**
	 * Gives access to the top level VOCLModel contained in the resource.
	 */
	private VOCLDiagram voclmodel = null;

	/**
	 * Returns the resource containing the VOCLmodel. Uses lazy initialization.
	 * @param path The path to the requested vocl file
	 * @return the requested resource
	 */
	public Resource getResource(IPath path) {
		if (resource == null) {
			ResourceSet resSet = getResourceSet();
			resource =
				resSet.getResource(
					URI.createPlatformResourceURI(path.toString()),
					true);
		}
		return resource;
	}

	/**
	 * Creates a resource to contain the VOCLModel. The resource file does not exist yet.
	 * @param path
	 * @return the resource
	 */
	private Resource createResource(IPath path) {
		if (resource == null) {
			ResourceSet resSet = getResourceSet();
			resource =
				resSet.createResource(
					URI.createPlatformResourceURI(path.toString()));
		}
		return resource;
	}

	/**
	 * Returns the resource set.
	 * @return the resource set
	 */
	private ResourceSet getResourceSet() {
		// Initialize the vocleditor package
		VOCLModelPackageImpl.init();
		// Register the XMI resource factory for the .vocl extension
		Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
		Map m = reg.getExtensionToFactoryMap();
		m.put("vocl", new XMIResourceFactoryImpl());
		// Obtain a new resource set
		return new ResourceSetImpl();
	}

	/**
	 * Returns the factory associated with the model.
	 * Object creation are made through that factory.
	 * @return the associated factory
	 */
	static public VOCLModelFactory getFactory() {
		if (voclmodelFactory == null) {
			// Access the factory (needed to create instances)
			Map registry = EPackage.Registry.INSTANCE;
			String voclmodelURI = VOCLModelPackage.eNS_URI;
			VOCLModelPackage voclmodelPackage =
				(VOCLModelPackage) registry.get(voclmodelURI);
			voclmodelFactory = voclmodelPackage.getVOCLModelFactory();
		}
		return voclmodelFactory;
	}

	/**
	 * Creates a new VOCLModel.
	 * @param path The path to the new file
	 * @return the new VOCLDiagram
	 */
	public VOCLDiagram createVOCLModel(IPath path) {
		createResource(path);
		// Create a new VOCLModel 
		Map registry = EPackage.Registry.INSTANCE;
		String voclmodelURI = VOCLModelPackage.eNS_URI;
		VOCLModelPackage nPackage = (VOCLModelPackage) registry.get(voclmodelURI);
		VOCLModelFactory nFactory = nPackage.getVOCLModelFactory();
		voclmodel = nFactory.createVOCLDiagram();
		resource.getContents().add(voclmodel);
		return voclmodel;
	}

	/**
	 * Loads the content of the model from the file.
	 * @param path
	 */
	public void load(IPath path) throws IOException {
		getResource(path);
		Map options = new HashMap();
		options.put(XMIResource.OPTION_DECLARE_XML, Boolean.TRUE);
		resource.load(options);
	}

	/**
	 * reloads the content of the model from the file.
	 * @param path
	 */
	public void reload(IPath path) throws IOException {
		getResource(path).unload();
		load(path);
	}

	/**
	 * Saves the content of the model to the file.
	 * @param path
	 */
	public void save(IPath path) throws IOException {
		//try {
			getResource(path);
			Map options = new HashMap();
			options.put(XMIResource.OPTION_DECLARE_XML, Boolean.TRUE);
			resource.save(options);
		//} catch (Exception e) {
		//	VOCLEditorPlugin.printDebug( "SAVING MODEL: caught exception: " + e );
		//}
	}
	
	/**
	 * Gets the top level VOCLmodel.
	 * @return the top {@link VOCLDiagram}
	 */
	public VOCLDiagram getModel() {
		if (null == voclmodel) {
			EList l = resource.getContents();
			Iterator i = l.iterator();
			while (i.hasNext()) {
				Object o = i.next();
				if (o instanceof VOCLDiagram)
					voclmodel = (VOCLDiagram) o;
			}
		}
		return voclmodel;
	}
}
